package org.itbikers.idea.ssh.ui.rundebug;

import com.intellij.execution.ExecutionException;
import com.intellij.execution.configurations.*;
import com.intellij.execution.runners.RunnerInfo;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.options.SettingsEditor;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.DefaultJDOMExternalizer;
import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.util.WriteExternalException;
import org.itbikers.idea.ssh.client.ConnectionData;
import org.itbikers.idea.ssh.resources.messages.SSHBundle;
import org.itbikers.idea.ssh.ui.toolwindow.SSHToolWindow;
import org.jdom.Element;

import java.util.Collection;

/**
 * An SSH configuration. Consists of: hostname, port, username & pwd. Gets persisted.
 * Should handle password security! todo
 *
 * @author mihai.slavcovici
 */
public class SSHRunConfiguration extends ModuleBasedConfiguration {

	private SSHRunDebugConfigType configType;

	public String hostname;
	public String port;
	public String username;
	public String password;
	public String command;

	public SSHRunConfiguration(String name, Project project, SSHConfigurationFactory configurationFactory) {
		super(name, new RunConfigurationModule(project, true), configurationFactory);
	}

	public SSHRunConfiguration(Project project, SSHConfigurationFactory configurationFactory) {
		this(SSHBundle.message("ssh.rundebug.config.name.default"), project, configurationFactory);
		configType = (SSHRunDebugConfigType) configurationFactory.getType();
	}

	public String getHostname() {
		return hostname;
	}

	public void setHostname(String hostname) {
		this.hostname = hostname;
	}

	public String getPort() {
		return port;
	}

	public void setPort(String port) {
		this.port = port;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getCommand() {
		return command;
	}

	public void setCommand(String command) {
		this.command = command;
	}

	public Collection<Module> getValidModules() {
		return getAllModules();
	}

	protected ModuleBasedConfiguration createInstance() {
		return new SSHRunConfiguration(getName(), getProject(), SSHRunDebugConfigType.getFactory());
	}


	/**
	 * Method invoked when run is called.
	 *
	 * @return this method returns null because {@link RunProfileState#execute()} is not invoked.
	 */
	public RunProfileState getState(DataContext context, RunnerInfo runnerInfo, RunnerSettings runnerSettings,
	                                ConfigurationPerRunnerSettings configurationSettings)
			throws ExecutionException {
		final SSHToolWindow toolWindow = SSHToolWindow.getInstance(getProject());
		final ConnectionData data = new ConnectionData();
		data.setCommand(this.command);
		data.setHostname(this.hostname);
		data.setPassword(this.password);
		data.setPort(Integer.parseInt(this.port));
		data.setSessionName(getName());
		data.setUsername(username);
		toolWindow.createTerminal(data);
		return null;
	}


	/**
	 * Checks if the configuration is valid.
	 *
	 * @throws RuntimeConfigurationException if configuration is not valid.
	 */
	@Override
	public void checkConfiguration() throws RuntimeConfigurationException {
		int port;
		try {
			port = Integer.parseInt(this.port);
		} catch (NumberFormatException e) {
			throw new RuntimeConfigurationException(SSHBundle.message("ssh.rundebug.error.port.format"));
		}
		if (1 > port && 65536 < port) {
			throw new RuntimeConfigurationException(SSHBundle.message("ssh.rundebug.error.port.range"));
		}
		if (this.isEmpty(this.hostname))
			throw new RuntimeConfigurationException(SSHBundle.message("ssh.rundebug.error.host.empty"));
		if (this.isEmpty(this.username))
			throw new RuntimeConfigurationException(SSHBundle.message("ssh.rundebug.error.username.empty"));
	}

	/**
	 * Returns true if the string is the emty string or null.
	 *
	 * @param s the string to test.
	 * @return true if the string is the emty string or null.
	 */
	protected boolean isEmpty(String s) {
		return s == null || "".equals(s);
	}


	public SettingsEditor<? extends RunConfiguration> getConfigurationEditor() {
		return new SSHRunDebugConfigForm();
	}

	public void setConfigType(SSHRunDebugConfigType configType) {
		this.configType = configType;
	}

	public ModuleBasedConfiguration clone() {
		SSHRunConfiguration clone = (SSHRunConfiguration) super.clone();
		clone.setConfigType(configType);
		clone.setHostname(hostname);
		clone.setPort(port);
		clone.setCommand(command);

		return clone;
	}

	public void readExternal(Element element)
			throws InvalidDataException {
		DefaultJDOMExternalizer.readExternal(this, element);
	}

	public void writeExternal(Element element)
			throws WriteExternalException {
		DefaultJDOMExternalizer.writeExternal(this, element);
	}
}
