//Project: 
//Date: Oct 30, 2006
//Author: andi
//File name: SSHToolWindow.java

package org.itbikers.idea.ssh.ui.toolwindow;

import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.util.IconLoader;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import org.itbikers.idea.ssh.client.ConnectionData;
import org.itbikers.idea.ssh.ui.PasswordDialog;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;


/**
 * The component that will be placed in the tool window. It has the following structure:
 * <table style="border: solid; ">
 * <tr>
 * <td width="20%" style="border: solid; ">buttons</td>
 * <td width="80%" style="vertical-align:middle; border:solid; text-align:center;">terminal</td>
 * </tr>
 * </table>
 * <p/>
 * There exists one {@link SSHToolWindow} per project.
 *
 * @author Andrei Diaconu-Muresan
 */
public class SSHToolWindow extends JPanel {
	/**
	 * Id of this tool window. Used as id in
	 * {@link com.intellij.openapi.wm.ToolWindowManager#registerToolWindow(String,javax.swing.JComponent,com.intellij.openapi.wm.ToolWindowAnchor)}
	 */
	public static final String TOOL_WINDOW_ID = "SSH";

	/**
	 * The tabbed pane that contains all the terminals.
	 */
	private JTabbedPane tabbedPane;
	/**
	 * The tool window manager. Used for removing the tool window after all terminal are closed.
	 */
	private ToolWindowManager toolWindowManager;

	/**
	 * Default constructor.
	 */
	private SSHToolWindow() {
		super(new BorderLayout());
		this.tabbedPane = new JTabbedPane();
		this.add(tabbedPane, BorderLayout.CENTER);
		final ActionManager actionManager = ActionManager.getInstance();
		final DefaultActionGroup actionGroup = new DefaultActionGroup("Close button group", false);
		final AnAction anAction = new AnAction() {
			public void actionPerformed(AnActionEvent e) {
				SSHToolWindow.this.closeCurrentTab();
			}
		};
		final Presentation presentation = anAction.getTemplatePresentation();
		presentation.setIcon(IconLoader.getIcon("/actions/cancel.png"));
		presentation.setDescription("Closes the current terminal");
		presentation.setText("Close");
		actionGroup.add(anAction);
		final ActionToolbar actionToolbar = actionManager.createActionToolbar("", actionGroup, false);
		this.add(actionToolbar.getComponent(), BorderLayout.WEST);
		tabbedPane.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				setFocus();
			}
		});
	}

	/**
	 * Sets the focus to the active pane.
	 */
	private void setFocus() {
		final TerminalTab terminalTab = (TerminalTab) this.tabbedPane.getSelectedComponent();
		if (terminalTab!=null){
			terminalTab.setFocus();
		}

	}

	/**
	 * Returns the tool window for the given project.
	 *
	 * @param project the current project.
	 * @return the tool window or null if error occures.
	 */
	public static SSHToolWindow getInstance(Project project) {
		final ToolWindowAnchor anchor = ToolWindowAnchor.BOTTOM;
		final ToolWindowManager toolWindowManager = ToolWindowManager.getInstance(project);
		final ToolWindow toolWindow = toolWindowManager.getToolWindow(SSHToolWindow.TOOL_WINDOW_ID);
		final SSHToolWindow sshToolWindow;
		if (toolWindow == null) {
			//no terminal tool window
			sshToolWindow = new SSHToolWindow();
			sshToolWindow.setToolWindowManager(toolWindowManager);
			toolWindowManager.registerToolWindow(SSHToolWindow.TOOL_WINDOW_ID, sshToolWindow, anchor);
			return sshToolWindow;
		} else {
			//terminal tool window exists; just add a new terminal
			if (toolWindow.getComponent() instanceof SSHToolWindow)
				return (SSHToolWindow) toolWindow.getComponent();
			else {
				Messages.showErrorDialog(project, "The tool window having the requested ID is wrong.", "Error");
				return null;
			}
		}

	}

	/**
	 * Creates a new terminal.
	 *
	 * @param connectionData the data. At this point the password may still be not set. Time to get it.
	 */
	public void createTerminal(ConnectionData connectionData) {
		boolean ok = this.ensureDataOk(connectionData);
		if (!ok)
			//user cancels
			return;
		final TerminalTab tab = new TerminalTab(connectionData);
		this.tabbedPane.addTab(connectionData.getSessionName(), tab);
		this.tabbedPane.setSelectedComponent(tab);
	}

	/**
	 * Checks for password, and if not set, asks.
	 *
	 * @param connectionData the connection data.
	 * @return true if the user provided a password and fale otherwise.
	 */
	private boolean ensureDataOk(ConnectionData connectionData) {
		if (connectionData.getPassword() == null ||
				connectionData.getPassword().length() == 0) {
			final PasswordDialog dialog = new PasswordDialog();
			dialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			dialog.pack();
			dialog.setLocationRelativeTo(null);
			dialog.setModal(true);
			dialog.setVisible(true);
			if (dialog.isOk()) {
				connectionData.setPassword(dialog.getPassword());
				return true;
			} else
				return false;
		}
		return true;

	}

	/**
	 * Sets the tool window manager.
	 *
	 * @param toolWindowManager the tool window manager.
	 */
	public void setToolWindowManager(ToolWindowManager toolWindowManager) {
		this.toolWindowManager = toolWindowManager;
	}

	/**
	 * Closes the current tab. If it is the last tab, the tool window is removed.
	 */
	private void closeCurrentTab() {
		final TerminalTab activeTab = (TerminalTab) this.tabbedPane.getSelectedComponent();
		this.tabbedPane.remove(activeTab);
		activeTab.close();
		final int tabcount = this.tabbedPane.getTabCount();
		if (tabcount == 0)
			this.close();
	}

	/**
	 * Closes the tool window.
	 */
	private void close() {
		this.toolWindowManager.unregisterToolWindow(TOOL_WINDOW_ID);
	}

}
